// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "scene/Scene.hpp"
#include "display/TextureRenderTarget.hpp"
#include "scene/components/display/Camera.hpp"

namespace Dawn {
  class SubSceneCameraAlign : public SceneItemComponent {
    protected:
      std::function<void()> evtCameraResized;
      std::function<void()> evtRenderResized;

      /**
       * Realigns the camera to match the render target quad.
       */
      void realign();

    public:
      // @optional
      StateProperty<Camera*> camera;
      // @optional
      StateProperty<TextureRenderTarget*> renderTarget;

      /**
       * Create the sub scene camera align component. This will align a camera
       * to match a render target quad (Refer SimpleRenderTargetQuad)
       * 
       * @param item Scene Item that this component belongs to.
       */
      SubSceneCameraAlign(SceneItem *item);

      void onStart() override;
  };
}