// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "util/random.hpp"
#include "EntityMove.hpp"
#include "EntityHealth.hpp"

namespace Dawn {
  enum EntityAIWalkMode {
    ENTITY_AI_WALK_MODE_FOLLOW_TARGET,
    ENTITY_AI_WALK_MODE_WANDER,
    ENTITY_AI_WALK_MODE_STAY
  };

  class EntityAIWalk : public SceneItemComponent {
    private:
      EntityMove *entityMove = nullptr;

      void updateWander(float_t delta);
      void updateFollowTarget(float_t delta);

    public:
      // Common
      // @optional
      enum EntityAIWalkMode mode = ENTITY_AI_WALK_MODE_STAY;

      // Wander Settings
      // @optional
      float_t wanderTimeToNextDecision = 0;
      // @optional
      glm::vec2 wanderDestination = glm::vec2(0, 0);
      // @optional
      glm::vec2 wanderTimeRandomRange = glm::vec2(0.5f, 2.0f);
      // @optional
      glm::vec2 wanderDistanceRandomRange = glm::vec2(12.0f, 12.0f);
      // @optional
      bool_t wanderSingleAxisOnly = true;
      // @optional
      float_t wanderDoNotMoveChance = 0.2f;

      // Follow Target Settings
      // @optional
      glm::vec2 followTarget = glm::vec2(0, 0);
      // @optional
      float_t followNearDistance = 2.0f;

      EntityAIWalk(SceneItem* item);
      std::vector<SceneItemComponent*> getDependencies() override;
      void onStart() override;
  };
}